/*
 * Copyright (c) 2007, 2008 ICM Uniwersytet Warszawski All rights reserved.
 * See LICENCE file for licencing information.
 *
 * Created on Sep 24, 2007
 * Author: K. Benedyczak <golbi@mat.umk.pl>
 */

package pl.edu.icm.unicore.uvos.wsclient.samlapi;

import java.net.MalformedURLException;
import java.net.URI;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Properties;

import eu.unicore.samly2.SAMLConstants;
import eu.unicore.samly2.assertion.Assertion;
import eu.unicore.samly2.elements.SAMLAttribute;
import eu.unicore.samly2.elements.Subject;
import eu.unicore.samly2.exceptions.SAMLParseException;
import eu.unicore.samly2.proto.AssertionResponse;
import eu.unicore.samly2.proto.AttributeQuery;
import eu.unicore.security.util.client.IAuthenticationConfiguration;

import pl.edu.icm.unicore.uvos.api.Attribute;
import pl.edu.icm.unicore.uvos.api.Group;
import pl.edu.icm.unicore.uvos.api.Identity;
import pl.edu.icm.unicore.uvos.api.exceptions.InvalidValueException;
import pl.edu.icm.unicore.uvos.wsapi.xmlbeans.SAMLQueryInterface;
import pl.edu.icm.unicore.uvos.wsapi.xmlbeans.SAMLXMLBeansMapper;
import pl.edu.icm.unicore.uvos.wsapi.xmlbeans.UnsupportedSAMLException;
import xmlbeans.org.oasis.saml2.assertion.AttributeStatementType;
import xmlbeans.org.oasis.saml2.assertion.AttributeType;
import xmlbeans.org.oasis.saml2.protocol.ResponseDocument;
import xmlbeans.pl.edu.icm.samlvo.attrext.RequestedGroupScopeDocument;

/**
 * This class should be used by UVOS consumers. It provides high level interface 
 * to query operations. 
 * <p>
 * For generic usage (e.g. not with UVOS server) there are some limitations:
 * <ul>
 * <li> Only string attribute values are supported.
 * <li> SAML responses with more than one AssertionStatement are unsupported.
 * </ul>
 * 
 * @author K. Benedyczak
 */
public class SAMLVOQueryClient extends SAMLVOClient implements SAMLVOQueryClientInterface
{
	private SAMLQueryInterface queryProxy;
	
	public SAMLVOQueryClient(String address, IAuthenticationConfiguration secProv, Properties settings) 
		throws SecuritySetupException, MalformedURLException
	{
		super(address, secProv, settings);
		initProxy();
	}

	public SAMLVOQueryClient(String address, IAuthenticationConfiguration secProv, 
			URI issuer, Properties settings)
		throws SecuritySetupException, MalformedURLException
	{
		super(address, secProv, issuer, settings);
		initProxy();
	}

	private void initProxy()
	{
		try
		{
			if (queryProxy == null)
				queryProxy = fact.getSAMLQueryClient(address);
		} catch (MalformedURLException e){}
	}
	
	public List<Attribute> getAttributes(Identity whose, String attributeName) 
		throws SAMLParseException, UnsupportedSAMLException, 
		InvalidSignatureException, InvocationException, SAMLErrorResponseException
	{
		return getAttributesGeneric(whose, attributeName, null, null, null);
	}

	public Attribute getAttribute(Identity whose, Group scope, String attributeName) 
		throws SAMLParseException, UnsupportedSAMLException, 
		InvalidSignatureException, InvocationException, SAMLErrorResponseException
	{
		List<Attribute> list = getAttributesGeneric(whose, 
				attributeName, null, scope, SAMLConstants.SCOPE_TYPE_ATTRIBUTE);
		if (list.size() == 0)
			return null;
		return list.get(0);
	}
	
	public List<Attribute> getAttributes(Identity whose, Attribute attribute) 
		throws SAMLParseException, UnsupportedSAMLException, 
		InvalidSignatureException, InvocationException, SAMLErrorResponseException
	{
		return getAttributesGeneric(whose, 
				attribute.getName(), 
				attribute.getValues(), null, null);
	}

	public Attribute getAttribute(Identity whose, Group scope, Attribute attribute) 
		throws SAMLParseException, UnsupportedSAMLException, 
		InvalidSignatureException, InvocationException, SAMLErrorResponseException
	{
		List<Attribute> list = getAttributesGeneric(whose, 
				attribute.getName(), 
				attribute.getValues(), scope, SAMLConstants.SCOPE_TYPE_ATTRIBUTE);
		if (list.size() == 0)
			return null;
		return list.get(0);
	}

	public List<Attribute> getAttributes(Identity whose) throws 
		SAMLParseException, UnsupportedSAMLException, InvalidSignatureException, 
		InvocationException, SAMLErrorResponseException
	{
		return getAttributesGeneric(whose, null, null, null, null);
	}
	
	public List<Attribute> getAttributes(Identity whose, Group scope) 
		throws SAMLParseException, UnsupportedSAMLException, 
		InvalidSignatureException, InvocationException, SAMLErrorResponseException
	{
		return getAttributesGeneric(whose, null, null, scope, null);
	}

	public List<Attribute> getSimpleScopedAttribute(Identity whose, Attribute attribute)
		throws SAMLParseException, UnsupportedSAMLException, InvalidSignatureException,
		InvocationException, SAMLErrorResponseException, InvalidValueException
	{
		List<String> vals = attribute.getValues();
		if (vals == null || vals.size() == 0)
			throw new InvalidValueException("No values set in attribute");
		for (String val: vals)
			if (!val.contains("@/"))
				throw new InvalidValueException("Values invalid: " + val);
		return getAttributesGeneric(whose, 
				attribute.getName(), 
				attribute.getValues(), null, SAMLConstants.SCOPE_TYPE_SIMPLE);
	}
	
	

	public Assertion getAssertion(Identity whose, String attributeName) throws SAMLParseException, UnsupportedSAMLException, InvalidSignatureException, InvocationException, SAMLErrorResponseException
	{
		return getAssertionGeneric(whose, attributeName, null, null, null);
	}

	public Assertion getAssertion(Identity whose) throws Exception
	{
		return getAssertionGeneric(whose, null, null, null, null);
	}

	public Assertion getAssertion(Identity whose, Group scope) throws SAMLParseException, UnsupportedSAMLException, InvalidSignatureException, InvocationException, SAMLErrorResponseException
	{
		return getAssertionGeneric(whose, null, null, scope, null);
	}

	public Assertion getAssertion(Identity whose, Attribute attribute) throws SAMLParseException, UnsupportedSAMLException, InvalidSignatureException, InvocationException, SAMLErrorResponseException
	{
		return getAssertionGeneric(whose, 
				attribute.getName(), 
				attribute.getValues(), null, null);
	}

	public Assertion getAssertionSimpleScoped(Identity whose, List<Attribute> attributes) throws SAMLParseException, UnsupportedSAMLException, InvalidSignatureException, InvocationException, SAMLErrorResponseException
	{
		return getAssertionGenericMulti(whose, attributes, null, 
			SAMLConstants.SCOPE_TYPE_SIMPLE);
	}

	public Assertion getAssertion(Identity whose, List<Attribute> attributes, Group scope) throws SAMLParseException, UnsupportedSAMLException, InvalidSignatureException, InvocationException, SAMLErrorResponseException
	{
		return getAssertionGenericMulti(whose, attributes, scope, null);
	}

	
	/*-********************************************************
	 * INTERNAL methods 
	 *-********************************************************/

	protected Assertion getAssertionGenericMulti(Identity whose, 
		List<Attribute> attributes, Group scope, String scopingType)
		throws SAMLParseException, UnsupportedSAMLException, 
		InvalidSignatureException, InvocationException, SAMLErrorResponseException
	{
		Subject subject = new Subject(whose.getValue(), 
				whose.getType().getName().toString()); 
		AttributeQuery attrQuery = new AttributeQuery(issuer, subject);
		String strScope = scope == null ? null : scope.toString();

		if (attributes != null && attributes.size() > 0)
		{
			SAMLAttribute []samlAttrs = new SAMLAttribute[attributes.size()];
			for (int i=0; i<attributes.size(); i++)
			{
				Attribute a = attributes.get(i);
				samlAttrs[i] = new SAMLAttribute(a.getName(), 
					SAMLConstants.AFORMAT_URI);
				if (scopingType == null && scope != null)
					scopingType = SAMLConstants.SCOPE_TYPE_ATTRIBUTE; 
				if (scopingType != null)
					samlAttrs[i].setScopingType(scopingType);
				List<String> attrValues = a.getValues();
				if (attrValues != null)
				{
					for (String aV: attrValues)
						addValue(samlAttrs[i], aV, scopingType, strScope);
				} else
				{
					if (scopingType != null && 
						!scopingType.equals(SAMLConstants.SCOPE_TYPE_NONE))
						samlAttrs[i].setAttributeWideScope(strScope);
				}
			}
			attrQuery.setAttributes(samlAttrs);
		}
		if (scope != null && (attributes == null || attributes.size() == 0))
		{
			RequestedGroupScopeDocument scopeDoc = RequestedGroupScopeDocument.
				Factory.newInstance();
			scopeDoc.addNewRequestedGroupScope().addGroup(strScope);
			attrQuery.setExtensions(scopeDoc);
		}
		
		signIfNeeded(attrQuery);

		ResponseDocument xmlRespDoc;
		
		//System.out.println(attrQuery.getDoc().xmlText(
		//		new XmlOptions().setSavePrettyPrint()));
		
		try
		{
			xmlRespDoc = queryProxy.attributeQuery(attrQuery.getDoc());
			lastResponse = xmlRespDoc;
		} catch (Exception e)
		{
			throw new InvocationException(e);
		}
		AssertionResponse resp = new AssertionResponse(xmlRespDoc);
		resp.parse();
		
		if (!resp.isStatusOK())
			throw new SAMLErrorResponseException(resp.getErrorMessage(),
					resp.getErrorStatus(), resp.getSubErrorStatus());

		checkResponseSignature(resp);

		Assertion[] assertions;
		try
		{
			assertions = resp.getAssertions();
		} catch (Exception e)
		{
			throw new SAMLParseException(e.toString());
		}
		
		if (assertions.length == 0)
			return null;
		if (assertions.length > 1)
			throw new UnsupportedSAMLException(
				"More than one assertion was returned. It is OK," +
				"however this implementation supports only " +
				"responses with a single assertion.");
		Assertion assertion = assertions[0];

		checkAssertionSignature(resp, assertion);

		return assertion;
	}
	
	protected Assertion getAssertionGeneric(Identity whose, 
		String attributeName, List<String> attrValues, Group scope, String scopingType)
		throws SAMLParseException, UnsupportedSAMLException, 
		InvalidSignatureException, InvocationException, SAMLErrorResponseException
	{
		List<Attribute> a = null;
		if (attributeName != null)
			a = Collections.singletonList(new Attribute(attributeName, attrValues, null)); 
		return getAssertionGenericMulti(whose, a, scope, scopingType);
	}
	
	
	protected List<Attribute> getAttributesGeneric(Identity whose, 
			String attributeName, List<String> attrValues, Group scope, String scopingType)
			throws SAMLParseException, UnsupportedSAMLException, 
			InvalidSignatureException, InvocationException, SAMLErrorResponseException
	{
		Assertion assertion = getAssertionGeneric(whose, attributeName, 
				attrValues, scope, scopingType);
		ArrayList<Attribute> ret = new ArrayList<Attribute>();
		if (assertion == null)
			return ret;

		AttributeStatementType []xmlAttrStatements = assertion.getAttributes();
		for (AttributeStatementType xmlAttrStatement: xmlAttrStatements)
		{
			AttributeType xmlAttrs[] = xmlAttrStatement.getAttributeArray();
			for (AttributeType xmlAttr: xmlAttrs)
			{
				List<Attribute> attr = SAMLXMLBeansMapper.map2APIAttributes(
						xmlAttr);
				ret.addAll(attr);
			}
		}

		return ret;
	}

	private void addValue(SAMLAttribute samlA, String value, String scopingType, String scope)
	{
		if (value != null)
		{
			if (!SAMLConstants.SCOPE_TYPE_ATTRIBUTE.equals(scopingType))
				samlA.addStringAttributeValue(value);
			else
				samlA.addScopedStringAttributeValue(value, scope);
		} else
		{
			if (scopingType.equals(SAMLConstants.SCOPE_TYPE_SIMPLE))
				samlA.addStringAttributeValue("@/" + 
						((scope != null) ? scope : ""));
			if (scopingType.equals(SAMLConstants.SCOPE_TYPE_ATTRIBUTE))
				samlA.addScopedStringAttributeValue(null, scope);
		}
	}

	@Override
	public Object getProxy()
	{
		return queryProxy;
	}
}
