/*
 * Copyright (c) 2007, 2008 ICM Uniwersytet Warszawski All rights reserved.
 * See LICENCE file for licencing information.
 *
 * Created on Apr 12, 2008
 * Author: K. Benedyczak <golbi@mat.umk.pl>
 */

package pl.edu.icm.uvos.test;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import junit.framework.TestCase;
import pl.edu.icm.unicore.uvos.api.Attribute;
import pl.edu.icm.unicore.uvos.api.Element;
import pl.edu.icm.unicore.uvos.api.Group;
import pl.edu.icm.unicore.uvos.api.Identity;
import pl.edu.icm.unicore.uvos.api.IdentityType;
import pl.edu.icm.unicore.uvos.client.util.ConfigHelper;
import pl.edu.icm.unicore.uvos.db.InitDB;
import pl.edu.icm.unicore.uvos.server.ComponentContainer;
import pl.edu.icm.unicore.uvos.server.VOServer;
import pl.edu.icm.unicore.uvos.util.Configuration;
import pl.edu.icm.unicore.uvos.wsclient.api.UVOSClientFactory;
import pl.edu.icm.unicore.uvos.wsclient.api.UVOSManagementExtInterface;
import pl.edu.icm.unicore.uvos.wsclient.samlapi.SAMLVOQueryClient;

/**
 * @author K. Benedyczak
 */
public class SAMLAttribute2Test extends TestCase
{
	private static final String UVOS_CONF = "src/test/resources/uvos/uvosServer.conf";

	private static boolean initDone = false;
	protected TestSecurityConfig conf;
	protected String addr;
	protected UVOSManagementExtInterface voMan;

	protected static Identity tested = new Identity(IdentityType.DN, 
		"CN=Example Client, OU=Example OU, O=Example O, L=Example City, ST=Example Province, C=PL");
	
	@Override
	protected void setUp()
	{
		try
		{
			Configuration c = new Configuration("src/test/resources/tlsTest.conf");
			conf = new TestSecurityConfig(c);
			addr = conf.getProtocol() + "://" + 
				conf.getServerHost() + ":" + conf.getServerPort();
			Configuration c2 = new Configuration("src/test/resources/functionalTest.conf");
			connectMan(c2);
		} catch (Throwable e)
		{
			System.err.println("Severe error when creating configuration");
			e.printStackTrace();
			fail();
		}

		if (initDone)
			return;
		try
		{
			InitDB.main(new String[]{UVOS_CONF, "-b"});
			ComponentContainer container = new ComponentContainer(UVOS_CONF);
			VOServer serverInstance = container.getComponent(VOServer.class);
			serverInstance.startup(container.getContainer());
			
			initAttributes();
		} catch (Throwable e)
		{
			System.err.println("Severe error when starting UVOS Web Server:");
			e.printStackTrace();
			fail();
		}
		initDone = true;
	}
	
	protected void connectMan(Configuration c) throws FileNotFoundException, IOException
	{				
		ConfigHelper manConf = new ConfigHelper(c);
		
		String addr = manConf.getProtocol() + "://" + 
			manConf.getServerHost() + ":" + manConf.getServerPort();
		UVOSClientFactory fact = new UVOSClientFactory(manConf, c);
		
		voMan = fact.getWSManagementClientHL(addr);
	}

	/**
	 * Adding attributes: 3 global (null value, 1 value, 2 values)
	 * and 3 scoped in /g/sub (where user is a member).
	 * 
	 * Therefore user receives the following effective attributes within different scopes:
	 *  3 global = 3
	 *  3 global x 2 scopes each = 6
	 *  3 scoped x 2 scopes each = 6
	 *  1 group attribute = 1
	 *  -----------------------------
	 *  16
	 */
	private void initAttributes() throws Exception
	{
		voMan.addIdentity(tested);
		
		Group g = new Group("/g");
		voMan.addGroup(null, g.getName());
		Group scope = voMan.addGroup(g, "sub");
		
		voMan.addToGroup(tested, scope);
		

		voMan.setAttribute(new Element(null, tested), new Attribute("global1", 
			null, null), false);
		voMan.setAttribute(new Element(null, tested), new Attribute("global2", 
			Collections.singletonList("v3"), null), false);
		List<String> vals = new ArrayList<String>();
		vals.add("v1");
		vals.add("v2");
		voMan.setAttribute(new Element(null, tested), new Attribute("global3", 
			vals, null), false);
		
		voMan.setAttribute(new Element(scope, tested), new Attribute("scoped1", 
			null, null), false);
		voMan.setAttribute(new Element(scope, tested), new Attribute("scoped2", 
			Collections.singletonList("v3"), null), false);
		voMan.setAttribute(new Element(scope, tested), new Attribute("scoped3", 
			vals, null), false);
	}
	
	public void testAll()
	{
		try
		{
			System.out.println(conf.doSSLAuthn);
			SAMLVOQueryClient client = new SAMLVOQueryClient(addr, conf, conf.getConfiguration());
			client.setDSigPolicy(true, true, null);
			
			List<Attribute> attrs = client.getAttributes(tested);
			if (attrs.size() != 16)
				fail("Should get 15 attributes, got " + attrs.size() + ": " + attrs);
		} catch (Exception e)
		{
			e.printStackTrace();
			fail("User was not authenticated");
		}
	}

	public void testIndividualOneValue()
	{
		try
		{
			System.out.println(conf.doSSLAuthn);
			SAMLVOQueryClient client = new SAMLVOQueryClient(addr, conf, conf.getConfiguration());
			client.setDSigPolicy(true, true, null);
			
			List<Attribute> attrs = client.getAttributes(tested, 
				new Attribute("global1", Collections.singletonList("v1"), null));
			if (attrs.size() != 0)
				fail("Should get 0 attributes, got " + attrs.size() + ": " + attrs);
			
			attrs = client.getAttributes(tested, 
				new Attribute("global2", Collections.singletonList("v3"), null));
			if (attrs.size() != 3)
				fail("Should get 3 attributes, got " + attrs.size() + ": " + attrs);

			attrs = client.getAttributes(tested, 
				new Attribute("global3", Collections.singletonList("v1"), null));
			if (attrs.size() != 3)
				fail("Should get 3 attributes, got " + attrs.size() + ": " + attrs);

			attrs = client.getAttributes(tested, 
				new Attribute("scoped1", Collections.singletonList("v1"), null));
			if (attrs.size() != 0)
				fail("Should get 0 attributes, got " + attrs.size() + ": " + attrs);

			attrs = client.getAttributes(tested, 
				new Attribute("scoped2", Collections.singletonList("v3"), null));
			if (attrs.size() != 2)
				fail("Should get 2 attributes, got " + attrs.size() + ": " + attrs);

			attrs = client.getAttributes(tested, 
				new Attribute("scoped3", Collections.singletonList("v1"), null));
			if (attrs.size() != 2)
				fail("Should get 2 attributes, got " + attrs.size() + ": " + attrs);
		} catch (Exception e)
		{
			e.printStackTrace();
			fail("User was not authenticated");
		}
	}


	public void testIndividualAllValues()
	{
		try
		{
			System.out.println(conf.doSSLAuthn);
			SAMLVOQueryClient client = new SAMLVOQueryClient(addr, conf, conf.getConfiguration());
			client.setDSigPolicy(true, true, null);
			
			List<Attribute> attrs = client.getAttributes(tested, 
				new Attribute("global1", null, null));
			if (attrs.size() != 3)
				fail("Should get 3 attributes, got " + attrs.size() + ": " + attrs);
			
			attrs = client.getAttributes(tested, 
				new Attribute("global2", null, null));
			if (attrs.size() != 3)
				fail("Should get 3 attributes, got " + attrs.size() + ": " + attrs);

			attrs = client.getAttributes(tested, 
				new Attribute("global3", null, null));
			if (attrs.size() != 3)
				fail("Should get 3 attributes, got " + attrs.size() + ": " + attrs);

			attrs = client.getAttributes(tested, 
				new Attribute("scoped1", null, null));
			if (attrs.size() != 2)
				fail("Should get 2 attributes, got " + attrs.size() + ": " + attrs);

			attrs = client.getAttributes(tested, 
				new Attribute("scoped2", null, null));
			if (attrs.size() != 2)
				fail("Should get 2 attributes, got " + attrs.size() + ": " + attrs);

			attrs = client.getAttributes(tested, 
				new Attribute("scoped3", null, null));
			if (attrs.size() != 2)
				fail("Should get 2 attributes, got " + attrs.size() + ": " + attrs);
		} catch (Exception e)
		{
			e.printStackTrace();
			fail("User was not authenticated");
		}
	}


	public void testScoped()
	{
		try
		{
			System.out.println(conf.doSSLAuthn);
			SAMLVOQueryClient client = new SAMLVOQueryClient(addr, conf, conf.getConfiguration());
			client.setDSigPolicy(true, true, null);
			
			List<Attribute> attrs = client.getAttributes(tested, 
				new Group("/g"));
			if (attrs.size() != 7)
				fail("Should get 7 attributes, got " + attrs.size() + ": " + attrs);

			attrs = client.getAttributes(tested, 
				new Group("/g/sub"));
			if (attrs.size() != 7)
				fail("Should get 7 attributes, got " + attrs.size() + ": " + attrs);
		} catch (Exception e)
		{
			e.printStackTrace();
			fail("User was not authenticated");
		}
	}



}
